function Gantt(data) {
	var self = this;

	data = data || {};

	self.task_table = data.task_table || "";

	self.element = data.element || "";
	self.parentId = data.parentId || "#grid-container";

	self.tableElement = null;

	self.minGanttSize = data.minGanttSize || $(self.element).width() * 0.6 || 1;

	self.fx = "";

	self.includeToday = data.includeToday || true;

	self.minDate = data.minDate || new Date();
	self.maxDate = data.maxDate || new Date();

	self.startMillis = data.startMillis || null;
	self.endMillis = data.endMillis || null;

	self.originalStartMillis = null;
	self.originalEndMillis = null;

	//self.zoomLevels = ["w", "m", "q", "s", "y"];
	self.zoomLevels = ["y", "s", "q","w","m"];	
	self.zoom = data.zoom || "y";

	self.init = function () {
		self.tableElement = self.create(self.zoom, self.startMillis, self.endMillis);
		$(self.element).append(self.tableElement);

		return self;
	};

	/*self.zoomGantt = function (isPlus) {
		console.log("*****self.zoom",self.zoom)
		var pos = this.zoomLevels.indexOf(self.zoom);
		var newPos = pos;
console.log("********Old Pos",newPos);
		if (isPlus) {
			newPos = (pos <= 0) ? this.zoomLevels.length - 1 : pos - 1;
		}
		else {
			newPos = (pos >= this.zoomLevels.length - 1) ? 0 : pos + 1;
		}
console.log("***newPos",isPlus,":",newPos);
		if (newPos != pos) {
			self.zoom = self.zoomLevels[newPos];
console.log("*****self.zoom",self.zoom)
			self.startMillis = self.originalStartMillis = new Date(self.minDate);
			self.endMillis = self.originalEndMillis = new Date(self.maxDate);

			self.refreshGantt();
		}

		return self;
	};*/
	self.zoomGantt = function (isPlus) {
		//console.log("*****self.zoom", self.zoom);
		var pos = this.zoomLevels.indexOf(self.zoom);
		var newPos;
	
		if (isPlus) {
			newPos = (pos === this.zoomLevels.length - 1) ? this.zoomLevels.length - 1 : pos + 1;
		} else {
			newPos = (pos === 0) ? 0 : pos - 1;
		}
	
		//console.log("***newPos", isPlus, ":", newPos);
	
		if (newPos !== pos) {
			self.zoom = self.zoomLevels[newPos];
			self.startMillis = self.originalStartMillis = new Date(self.minDate);
			self.endMillis = self.originalEndMillis = new Date(self.maxDate);
	
			self.refreshGantt();
		}
	
		return self;
	};

	var labels = {
		"GANT_QUARTER_SHORT1": "1st Quarter",
		"GANT_QUARTER_SHORT2": "2nd Quarter",
		"GANT_QUARTER_SHORT3": "3rd Quarter",
		"GANT_QUARTER_SHORT4": "4th Quarter",
		"GANT_SEMESTER_SHORT1": "1st Semester",
		"GANT_SEMESTER_SHORT2": "2nd Semester"
	};
	var createHeadCell = function (lbl, span, additionalClass) {
		var th = $("<th>").html(labels[lbl] || lbl).attr("colSpan", span);
		if (additionalClass)
			th.addClass(additionalClass);
		return th;
	};

	var createBodyCell = function (span, isEnd, additionalClass, title) {
		var ret = $("<td>").html("&nbsp;").attr("colSpan", span).attr("title", title).addClass("ganttBodyCell");
		if (isEnd)
			ret.addClass("end");
		if (additionalClass)
			ret.addClass(additionalClass);
		return ret;
	};

	var getPeriod = function (zoomLevel, stMil, endMillis) {
		var start = new Date(stMil),
			end = new Date(endMillis);

		switch (zoomLevel) {
			case "d":	// Reset hours
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);
				break;
			case "w":	// Reset day of week
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);

				start.setFirstDayOfThisWeek();
				end.setFirstDayOfThisWeek();
				end.setDate(end.getDate() + 6);
				break;
			case "m":	// Reset day of month
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);

				start.setDate(1);
				end.setDate(1);
				end.setMonth(end.getMonth() + 1);
				end.setDate(end.getDate() - 1);
				break;
			case "q":	// Reset to quarter
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);
				start.setDate(1);
				start.setMonth(Math.floor(start.getMonth() / 3) * 3);
				end.setDate(1);
				end.setMonth(Math.floor(end.getMonth() / 3) * 3 + 3);
				end.setDate(end.getDate() - 1);
				break;
			case "s":	// Reset to semester
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);
				start.setDate(1);

				start.setMonth(Math.floor(start.getMonth() / 6) * 6);
				end.setDate(1);
				end.setMonth(Math.floor(end.getMonth() / 6) * 6 + 6);
				end.setDate(end.getDate() - 1);
				break;
			case "y":	// Reset to year -> gen
				start.setHours(0, 0, 0, 0);
				end.setHours(23, 59, 59, 999);

				start.setDate(1);
				start.setMonth(0);

				end.setDate(1);
				end.setMonth(12);
				end.setDate(end.getDate() - 1);
				break;
		}

		return {
			start: start.getTime(),
			end: end.getTime()
		};
	};

	var computedTableWidth;

	self.createGantt = function (zoom, startPeriod, endPeriod) {
		zoom = zoom || self.zoom;

		var tr1 = $("<tr/>").addClass("ganttHead1");
		var tr2 = $("<tr/>").addClass("ganttHead2");
		var trBody = $("<tr/>").addClass("ganttBody");

		var iterate = function (renderFunction1, renderFunction2) {
			var start;

			// Loop for header1
			start = new Date(startPeriod);
			while (start.getTime() <= endPeriod) {
				renderFunction1(start);
			}

			// Loop for header2
			start = new Date(startPeriod);
			while (start.getTime() <= endPeriod) {
				renderFunction2(start);
			}
		};

		var drawToday = function (box) {
			if (new Date().getTime() > self.startMillis && new Date().getTime() < self.endMillis) {
				var x = Math.round(((new Date().getTime()) - self.startMillis) * self.fx);
				$(".ganttToday", box).remove();
				var today = $("<div>").addClass("ganttToday").css("left", x);
				box.append(today);
			}
		};

		if (zoom == "y") {
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24 * 180)) * 100); //180gg = 1 sem = 100px
			iterate(function (date) {
				tr1.append(createHeadCell(date.format("YYYY"), 2));
				date.setFullYear(date.getFullYear() + 1);
			}, function (date) {
				var sem = (Math.floor(date.getMonth() / 6) + 1);
				tr2.append(createHeadCell("GANT_SEMESTER_SHORT" + sem, 1));
				trBody.append(createBodyCell(1, sem == 2));
				date.setMonth(date.getMonth() + 6);
			});
		}
		else if (zoom == "s")	// Semester
		{
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24 * 90)) * 100); //90gg = 1 quarter = 100px
			iterate(function (date) {
				var end = new Date(date.getTime());
				end.setMonth(end.getMonth() + 6);
				end.setDate(end.getDate() - 1);
				tr1.append(createHeadCell(date.format("MMM") + " - " + end.format("MMM yyyy"), 2));
				date.setMonth(date.getMonth() + 6);
			}, function (date) {
				var quarter = (Math.floor(date.getMonth() / 3) + 1);
				tr2.append(createHeadCell("GANT_QUARTER_SHORT" + quarter, 1));
				trBody.append(createBodyCell(1, quarter % 2 == 0));
				date.setMonth(date.getMonth() + 3);
			});
		}
		else if (zoom == "q") // Quarter
		{
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24 * 30)) * 300); //1 month= 300px
			iterate(function (date) {
				var end = new Date(date.getTime());
				end.setMonth(end.getMonth() + 3);
				end.setDate(end.getDate() - 1);
				tr1.append(createHeadCell(date.format("MMM") + " - " + end.format("MMM yyyy"), 3));
				date.setMonth(date.getMonth() + 3);
			}, function (date) {
				var lbl = date.format("MMM");
				tr2.append(createHeadCell(lbl, 1));
				trBody.append(createBodyCell(1, date.getMonth() % 3 == 2));
				date.setMonth(date.getMonth() + 1);
			});
		}
		else if (zoom == "m")	// Month
		{
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24 * 1)) * 60); //1 day= 20px
			iterate(function (date) {
				var sm = date.getTime();
				date.setMonth(date.getMonth() + 1);
				var daysInMonth = parseInt((date.getTime() - sm) / (3600000 * 24));
				tr1.append(createHeadCell(new Date(sm).format("MMMM yyyy"), daysInMonth)); //spans mumber of dayn in the month
			}, function (date) {
				tr2.append(createHeadCell(date.format("DD MMM"), 1, date.isLeave() ? "holyH" : null));
				var nd = new Date(date.getTime());
				nd.setDate(date.getDate() + 1);
				var addClass = "";
				if (date.isToday()) {
					addClass += " today";
				}
				if (date.isLeave()) {
					addClass += " holy";
				}
				trBody.append(createBodyCell(1, nd.getDate() == 1, addClass, nd.format(Otime.dateFormat)));
				date.setDate(date.getDate() + 1);
			});
		}
		else if (zoom == "w")	//week
		{
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24)) * 30); //1 day= 30px

			iterate(function (date) {
				var end = new Date(date.getTime());
				end.setDate(end.getDate() + 6);
				tr1.append(createHeadCell(date.format("MMM DD") + " - " + end.format("MMM DD'YYYY"), 7));
				date.setDate(date.getDate() + 7);
			},
				function (date) {
					tr2.append(createHeadCell(date.format("DDDD").substr(0, 1), 1, date.isLeave() ? "holyH" : null));
					trBody.append(createBodyCell(1, date.getDay() % 7 == (Date.firstDayOfWeek + 6) % 7, date.isLeave() ? "holy" : null, date.format(Otime.dateFormat)));
					date.setDate(date.getDate() + 1);
				});
		}
		else if (zoom == "d")	// Days
		{
			computedTableWidth = Math.floor(((endPeriod - startPeriod) / (3600000 * 24)) * 200); //1 day= 200px

			iterate(function (date) {
				tr1.append(createHeadCell(date.format("DDDD DD MMMM YYYY"), 4, date.isLeave() ? "holyH" : null));
				date.setDate(date.getDate() + 1);
			}, function (date) {
				tr2.append(createHeadCell(date.format("HH"), 1, date.isLeave() ? "holyH" : null));
				trBody.append(createBodyCell(1, date.getHours() > 17, date.isLeave() ? "holy" : null, nd.format(Otime.dateFormat)));
				date.setHours(date.getHours() + 6);
			});
		}
		else {
			console.error("Wrong level " + zoom);
		}

		computedTableWidth = Math.max(computedTableWidth, self.minGanttSize);

		// console.log(zoom, computedTableWidth);

		var box = $(self.parentId + " #ganttC");
		box.css({ width: computedTableWidth });

		var box1 = $("#user-grid-container #ganttC");
		box1.css({ width: computedTableWidth });

		var ganttHead = $("#ghTable"),
			ganttBody = $(self.parentId + " #gbTable"),
			thead = $("<thead/>"),
			tbody = $("<tbody/>");

		$("#ganttHead, #ganttBody, #scr3 .bar").css({ width: computedTableWidth });
		ganttHead.html("").append(thead);
		thead.append(tr1).append(tr2);
		ganttBody.html("").append(tbody).height($(".task-tbl", $(self.task_table)).innerHeight());
		tbody.append(trBody);

		//		table.height(self.master.editor.element.height());

		// Compute scalefactor fx
		self.fx = (computedTableWidth / (endPeriod - startPeriod));

		// Table Draw End

		var tlg = $(self.parentId + " #ganttC .task-list-group");

		//		tlg.css({
		//			top: $(".tbody", $(self.task_table)).position().top
		//		});

		// Create link container
		var links = $(".ganttLinks", tlg);
		links.css({
			width: computedTableWidth
		});

		drawToday(box);
		drawToday(box1);

		return box;
	};

	self.create = function (zoom, originalStartmillis, originalEndMillis) {
		// if include today synch extremes
				//console.log(originalStartmillis, originalEndMillis);
		if (self.includeToday) {
			var today = new Date().getTime();
			originalStartmillis = originalStartmillis > today ? today : originalStartmillis;
			originalEndMillis = originalEndMillis < today ? today : originalEndMillis;
		}

		// get best dimension for gantt
		// this is enlarged to match complete periods basing on zoom level
		var period = getPeriod(zoom, originalStartmillis, originalEndMillis);
		// Real dimension of gantt
		self.startMillis = new Date(period.start);
		self.endMillis = new Date(period.end);
		// Minimal dimension required by user or by task duration
		self.originalStartMillis = new Date(originalStartmillis);
		self.originalEndMillis = new Date(originalEndMillis);

		return self.createGantt(zoom, period.start, period.end);
	};

	self.resizeTable = function (task) {
		var refresh = false;

		if (task.form.duration() <= 0 || !task.form.start_date() || task.form.start_date() == "0000-00-00") {
			return true;
		}

		if (new Date(task.form.start_date()).getTime() < new Date(self.originalStartMillis).getTime()) {
			//			console.log("Date lower then current start date.", new Date(self.originalStartMillis), new Date(task.form.start_date()));
			self.startMillis = new Date(task.form.start_date()).getWeekStart();
			self.originalStartMillis = new Date(self.startMillis);
			refresh = true;
		}
		if (new Date(task.form.finish_date()).getTime() > new Date(self.originalEndMillis).getTime()) {
			//			console.log("Date higher then current finish date.", new Date(self.originalEndMillis), new Date(task.form.finish_date()));
			self.endMillis = new Date(task.form.finish_date()).getWeekEnd();
			self.originalEndMillis = self.endMillis;
			refresh = true;
		}

		if (refresh) {
			self.refreshGantt(task.form.start_date());
			return true;
		}

		$("table", self.tableElement).height(self.task_table.height());
		$("#splitt-bar").height(self.task_table.height());
		return false;
	};

	var getHandles = function (predec) {
		if (!predec) {
			return "e,w";
		}

		predec = predec.toUpperCase().split(",");

		//		FS (default), SS - start date read only
		//		SF, FF - finish date read only

		var handle = [];

		for (var i = 0; i < predec.length; i++) {
			var newHandle = "";

			if (predec[i].search("FS") < 0 && predec[i].search("SS") < 0) {
				newHandle = "w";
			}

			if (predec[i].search("SF") < 0 && predec[i].search("FF") < 0) {
				newHandle = "e"; // remove finish date change.
			}

			newHandle = newHandle || "w";

			if (handle.indexOf(newHandle) < 0) {
				handle.push(newHandle);
			}
		}

		return handle.join(",") || "w";
	};

	var toolTip = function (elem, disable) {
		elem.unbind("mousemove");
		if (disable) {
			return true;
		}
		$(self.parentId + " #gantt-tooltip").show();
		elem.mousemove(function (event) {
			$(self.parentId + " #gantt-tooltip").css({
				left: event.pageX + 10,
				top: event.pageY + 10,
				opacity: 0.8
			});
		});
	};
	/*
		var old_getToolTipHtml	= function(task){
			var html	 = '<table cellpadding="3" class="table">';
				html	+= '	<thead>';
				html	+= '		<tr><th colspan="3"><span class="name">' + task.form.name() + '</span></th></tr>';
				html	+= '	</thead>';
				html	+= '	<tbody>';
				html	+= '		<tr><td>' + new Date(task.form.start_date()).format(Otime.dateFormat) + '</td><td class="ac">&nbsp;-&nbsp;</td><td>' + new Date(task.form.finish_date()).format(Otime.dateFormat) + '</td></tr>';
				html	+= '		<tr><td colspan="2">Duration</td><td>' + task.form.duration() + ' Days</td></tr>';
			if (task.form.estimated_hours()-0)
				html	+= '		<tr><td colspan="2">Estimated</td><td>' + task.form.estimated_hours() + '</td></tr>';
			if (task.form.percentage_complete()-0)
				html	+= '		<tr><td colspan="2">Progress</td><td>' + task.form.percentage_complete() + '%</td></tr>';
				html	+= '	</tbody>';
				html	+= '</table>';
	
			return html;
		};
	*/
	var getToolTipHtml = function (task) {
		var html = task.form.name() + '\n';
		html += new Date(task.form.start_date()).format(Otime.dateFormat) + '\t-\t' + new Date(task.form.finish_date()).format(Otime.dateFormat) + '\n';
		html += 'Duration\t\t:\t' + task.form.duration() + ((task.form.duration() - 0) == 1 ? ' Day\n' : ' Days\n');
		if (task.form.estimated_hours() - 0)
			html += 'Estimated\t\t:\t' + task.form.estimated_hours() + ((task.form.estimated_hours() - 0) > 1 ? ' Hours\n' : ' Hour\n');
		if (task.form.percentage_complete() - 0)
			html += 'Progress\t\t:\t' + task.form.percentage_complete() + '%';

		return html;
	};

	var getHoursToolTipHtml = function (title) {
		var hourstitle = [];
		if (title) {
			hourstitle = title.split(",");
		}
		var html = "";
		for (let index = 0; index < hourstitle.length; index++) {
			if (index > 0) {
				html += '\n';
			}
			html += hourstitle[index];
		}
		return html;
	};
	self.drawUserHour = function (hour, elem) {		
		var endPeriod = new Date(hour.entry_date).getTime();
		var startPeriod = new Date(self.startMillis).getTime();
		var boxPosition = Math.floor(((endPeriod - startPeriod) / (3600000 * 24 * 1)) * 60);
		elem.style.left = (boxPosition + 3) + "px";		
		elem.title = getHoursToolTipHtml(hour.title);
	}	

	self.drawTask = function (task, isInit) {
		var editorRow = $("tr.task_" + task.uniqueId());
		var taskBox = $("div.taskG_" + task.uniqueId() + " .taskBox");

		if (!taskBox.get(0)) {
			console.log("Task Box not available.", taskBox);
			return false;
		}
		taskBox.attr("title", getToolTipHtml(task));
		/*
				// Tooltip changes.
				taskBox.hover(function(e){
					if (e.type == "mouseleave")
					{
						$("#gantt-tooltip").css({ opacity:0, left:0, top:0 });
						toolTip(taskBox, true);
						return true;
					}
		
					$("#gantt-tooltip").html(getToolTipHtml(task));
		
					toolTip(taskBox);
				});
		*/
		if (task.form.is_parent()) {
			//			self.resizeTaskBox(task);
			return true;
		}

		taskBox.resizable({
			//			grid: one_day_size,
			//			grid: [one_day_size, one_day_size],
			handles: getHandles(task.form.predecessors()),
			resize: function (event, ui) {
				$(self.parentId + " .taskLabel", taskBox).css("width", ui.position.left);
				event.stopImmediatePropagation();
				event.stopPropagation();
			},
			stop: function (event, ui) {
				var s = new Date(new Date(Math.round((ui.position.left / self.fx) + self.startMillis.getTime())).setHours(0, 0, 0, 0));
				var e = new Date(new Date(Math.round(((ui.position.left + ui.size.width) / self.fx) + self.startMillis.getTime())).setHours(0, 0, 0, 0));

				//				var s	= Math.round((ui.position.left / self.fx) + self.startMillis.getTime());
				//				task.form.start_date(new Date(s).format("YYYY/MM/DD"));

				var end = new Date(task.form.start_date()).getTime();
				var start = new Date(task.form.finish_date()).getTime();

				if (end != e.getTime()) {
					//					task.form.duration((end - start)/864E5);
					task.form.finish_date(e.format("YYYY/MM/DD"));
					if (self.maxDate.getTime() < e.getTime()) {
						self.maxDate = e;
					}
				}

				if (start != s.getTime()) {
					//					task.form.duration((end - start)/864E5);

					task.startFlag = true;
					task.form.start_date(s.format("YYYY/MM/DD"));
					task.startFlag = false;
					if (self.minDate.getTime() > s.getTime()) {
						self.minDate = s;
					}
				}

				//				task.du_flag	= true;
				//				task.du_flag	= false;

				taskBox.attr("title", getToolTipHtml(task));
				//				$("#gantt-tooltip").html(getToolTipHtml(task));

				//				toolTip(taskBox);
			}
		});

		if (!task.form.predecessors()) {
			taskBox.css("position", "absolute").draggable({
				axis: 'x',
				drag: function (event, ui) {
					$(".taskLabel", taskBox).css("width", ui.position.left);
				},
				stop: function (event, ui) {
					var s = Math.round((ui.position.left / self.fx) + self.startMillis.getTime());
					task.form.start_date(new Date(s).format("YYYY/MM/DD"));
				}
			});
		}
		//
		//		if (!isInit)
		//			self.resizeTaskBox(task);
	};

	self.resizeTaskBox = function (task) {
		var editorRow = $("tr.task_" + task.uniqueId());
		var taskBox = $("div.taskG_" + task.uniqueId() + " .taskBox");
		var x = 0;

		var sDate = task.form.start_date(),
			fDate = task.form.finish_date();

		var isZero = (!sDate || !fDate) ? true : false;

		sDate = new Date(sDate);
		fDate = new Date(fDate);
		x = Math.round((sDate.getTime() - self.startMillis.getTime()) * self.fx);
		if (task.form.is_milestone() && !(task.form.duration() - 0)) {
			x = Math.round((fDate.getTime() - self.startMillis.getTime()) * self.fx);
		}

		var width = Math.round((((new Date(fDate).setHours(23, 59, 59, 999)) - new Date(sDate).setHours(0, 0, 0, 0))) * self.fx);
		//		var width	= Math.round((((new Date(task.form.finish_date())) - new Date(task.form.start_date()).setHours(0, 0, 0, 0))) * self.fx);

		taskBox.css({
			left: x,
			width: isZero ? 0 : width
		});

		if (taskBox.length > 1) {
			console.log("Error : Duplicate Task Box try changing task self.uniqueId value random : ", taskBox.length, taskBox);
			return;
		}

		// remove drag event and resize event for parent items.
		if (task.form.is_parent() || (task.children() && task.children().length > 0)) {
			if (taskBox.data("uiResizable")) {
				taskBox.resizable("destroy");
			}

			if (taskBox.data("uiDraggable")) {
				taskBox.draggable("destroy");
			}

			return true;
		}

		if (isZero && taskBox.data("uiResizable")) {
			taskBox.resizable("destroy");
			return true;
		}

		if (taskBox.data("uiResizable")) {			
			taskBox.resizable("option", "handles", getHandles(task.form.predecessors()));
		}
	};

	self.clearLinks = function () {
		self.element.find(".ganttLinks").empty();
	};

	var peduncolusSize = 10;
	var lineSize = 2;

	var HLine = function (width, top, left) {
		var hl = $("<div>").addClass("taskDepLine");
		hl.css({
			height: lineSize,
			left: left,
			width: width,
			top: top - lineSize / 2
		});
		return hl;
	};
	var VLine = function (height, top, left) {
		var vl = $("<div>").addClass("taskDepLine");
		vl.css({
			height: height,
			left: left - lineSize / 2,
			width: lineSize,
			top: top
		});
		return vl;
	};
	var getParent = function (item) {
		if (!item.parent) {
			return false;
			//				return item;
		}
		item = item.parent();
		if (item && item.uniqueId && $("#ganttC .taskG_" + item.uniqueId()).get(0)) {
			return item;
		}
		else {
			return getParent(item);
		}
	};
	var buildRect = function (item) {
		var elem1 = $("#ganttC .taskG_" + item.uniqueId());
		if (!elem1.get(0)) {
			item = getParent(item);
			if (!item) {
				return false;
			}
			elem1 = $("#ganttC .taskG_" + item.uniqueId());
		}
		var elem2 = $(".taskBox", elem1);

		var rect = elem1.position();
		rect.left = elem2.position().left;
		rect.width = elem2.width();
		rect.height = elem2.height();
		return rect;
	};
	var getNodeBox = function (from, to) {
		var node_id = "link_" + from.form.task_number() + "_" + to.form.task_number();
		if ($("#" + node_id).get(0)) {
			$("#" + node_id).remove();
		}

		var ndo = $("<div>").attr({
			id: node_id,
			from: from.form.task_number(),
			to: to.form.task_number()
		}).addClass("link-grp");

		return ndo;
	};

	var drawStartToFinish = function (ndo, rectFrom, rectTo, peduncolusSize) {
		var left, top;

		var currentX = rectFrom.left + rectFrom.width;
		var currentY = rectFrom.height / 2 + rectFrom.top;

		var useThreeLine = (currentX + 2 * peduncolusSize) < rectTo.left;

		if (!useThreeLine) {
			// L1
			if (peduncolusSize > 0) {
				var l1 = new HLine(peduncolusSize, currentY, currentX);
				currentX = currentX + peduncolusSize;
				ndo.append(l1);
			}

			// L2
			var l2_4size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2)) / 2;
			var l2;
			if (l2_4size < 0) {
				l2 = new VLine(-l2_4size, currentY + l2_4size, currentX);
			}
			else {
				l2 = new VLine(l2_4size, currentY, currentX);
			}
			currentY = currentY + l2_4size;

			ndo.append(l2);

			// L3
			var l3size = rectFrom.left + rectFrom.width + peduncolusSize - (rectTo.left - peduncolusSize);
			currentX = currentX - l3size;
			var l3 = new HLine(l3size, currentY, currentX);
			ndo.append(l3);

			// L4
			var l4;
			if (l2_4size < 0) {
				l4 = new VLine(-l2_4size, currentY + l2_4size, currentX);
			}
			else {
				l4 = new VLine(l2_4size, currentY, currentX);
			}
			ndo.append(l4);

			currentY = currentY + l2_4size;

			// L5
			if (peduncolusSize > 0) {
				var l5 = new HLine(peduncolusSize, currentY, currentX);
				currentX = currentX + peduncolusSize;
				ndo.append(l5);
			}
		}
		else {
			//L1
			var l1_3Size = (rectTo.left - currentX) / 2;
			var l1 = new HLine(l1_3Size, currentY, currentX);
			currentX = currentX + l1_3Size;
			ndo.append(l1);

			//L2
			var l2Size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2));
			var l2;
			if (l2Size < 0) {
				l2 = new VLine(-l2Size, currentY + l2Size, currentX);
			}
			else {
				l2 = new VLine(l2Size, currentY, currentX);
			}
			ndo.append(l2);

			currentY = currentY + l2Size;

			//L3
			var l3 = new HLine(l1_3Size, currentY, currentX);
			currentX = currentX + l1_3Size;
			ndo.append(l3);
		}

		//arrow
		var arr = $("<img src='" + Otime.baseUrl + "/assets/img/linkArrow.png'>").css({
			position: 'absolute',
			top: rectTo.top + rectTo.height / 2 - 5,
			left: rectTo.left - 5
		});

		ndo.append(arr);

		return ndo;
	};

	var drawStartToStart = function (ndo, rectFrom, rectTo, peduncolusSize) {
		var left, top;

		var currentX = rectFrom.left;
		var currentY = rectFrom.height / 2 + rectFrom.top;

		var useThreeLine = (currentX + 2 * peduncolusSize) < rectTo.left;

		if (!useThreeLine) {
			// L1
			if (peduncolusSize > 0) {
				var l1 = new HLine(peduncolusSize, currentY, currentX - peduncolusSize);
				currentX = currentX - peduncolusSize;
				ndo.append(l1);
			}

			// L2
			var l2_4size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2)) / 2;
			var l2;
			if (l2_4size < 0) {
				l2 = new VLine(-l2_4size, currentY + l2_4size, currentX);
			}
			else {
				l2 = new VLine(l2_4size, currentY, currentX);
			}
			currentY = currentY + l2_4size;

			ndo.append(l2);

			// L3
			var l3size = (rectFrom.left - peduncolusSize) - (rectTo.left - peduncolusSize);
			currentX = currentX - l3size;
			var l3 = new HLine(l3size, currentY, currentX);
			ndo.append(l3);

			// L4
			var l4;
			if (l2_4size < 0) {
				l4 = new VLine(-l2_4size, currentY + l2_4size, currentX);
			}
			else {
				l4 = new VLine(l2_4size, currentY, currentX);
			}
			ndo.append(l4);

			currentY = currentY + l2_4size;

			// L5
			if (peduncolusSize > 0) {
				var l5 = new HLine(peduncolusSize, currentY, currentX);
				currentX = currentX + peduncolusSize;
				ndo.append(l5);
			}
		}
		else {
			//L1

			var l1 = new HLine(peduncolusSize, currentY, currentX - peduncolusSize);
			currentX = currentX - peduncolusSize;
			ndo.append(l1);

			//L2
			var l2Size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2));
			var l2;
			if (l2Size < 0) {
				l2 = new VLine(-l2Size, currentY + l2Size, currentX);
			}
			else {
				l2 = new VLine(l2Size, currentY, currentX);
			}
			ndo.append(l2);

			currentY = currentY + l2Size;

			//L3

			var l3 = new HLine(peduncolusSize + (rectTo.left - rectFrom.left), currentY, currentX);
			currentX = currentX + peduncolusSize + (rectTo.left - rectFrom.left);
			ndo.append(l3);
		}

		//arrow
		var arr = $("<img src='" + Otime.baseUrl + "/assets/img/linkArrow.png'>").css({
			position: 'absolute',
			top: rectTo.top + rectTo.height / 2 - 5,
			left: rectTo.left - 5
		});

		ndo.append(arr);

		return ndo;
	};
	var drawFinishToFinish = function (ndo, rectFrom, rectTo, peduncolusSize) {
		var left, top;

		var currentX = rectFrom.left;
		var currentY = rectFrom.height / 2 + rectFrom.top;

		var fromRight = (rectFrom.left + rectFrom.width);
		var toRight = (rectTo.left + rectTo.width);

		var l1_size = (fromRight > toRight) ? 0 : (toRight - fromRight);
		var l3_size = (fromRight > toRight) ? (fromRight - toRight) : 0;

		//L1
		var l1 = new HLine(l1_size + peduncolusSize, currentY, currentX + rectFrom.width);
		currentX = currentX + rectFrom.width + l1_size + peduncolusSize;
		ndo.append(l1);

		//L2
		var l2Size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2));
		var l2;
		if (l2Size < 0) {
			l2 = new VLine(-l2Size, currentY + l2Size, currentX);
		}
		else {
			l2 = new VLine(l2Size, currentY, currentX);
		}
		ndo.append(l2);

		currentY = currentY + l2Size;

		//L3
		var l3 = new HLine(l3_size + peduncolusSize, currentY, currentX - (l3_size + peduncolusSize));
		//				currentX	= currentX + peduncolusSize + (rectTo.left - rectFrom.left);
		ndo.append(l3);

		//arrow
		var arr = $("<img src='" + Otime.baseUrl + "/assets/img/linkArrow.png'>").css({
			position: 'absolute',
			top: rectTo.top + rectTo.height / 2 - 5,
			left: rectTo.left + rectTo.width
		});

		ndo.append(arr);

		return ndo;
	};

	var drawFinishToStart = function (ndo, rectFrom, rectTo, peduncolusSize) {
		var left, top;

		var currentX = rectFrom.left;
		var currentY = (rectFrom.height / 2) + rectFrom.top;

		var useThreeLine = (currentX + rectFrom.width + peduncolusSize) < rectTo.left;
		// console.log("useThreeLine : ", useThreeLine);
		if (!useThreeLine) {
			// L1
			currentX = currentX + rectFrom.width;
			var l1 = new HLine(peduncolusSize, currentY, currentX);
			ndo.append(l1);

			// L2
			var l2_4size = (((rectTo.top + (rectTo.height / 2)) - (rectFrom.top + (rectFrom.height / 2))) / 2) + 3;

			var l2;
			if (l2_4size < 0) {
				l2 = new VLine(-l2_4size, currentY + l2_4size, currentX + peduncolusSize);
			}
			else {
				l2 = new VLine(l2_4size, currentY, currentX + peduncolusSize);
			}
			currentY = currentY + l2_4size;

			ndo.append(l2);

			// L3
			var l3size = (rectFrom.left + rectFrom.width + peduncolusSize) - (rectTo.left - peduncolusSize);

			currentX = currentX - l3size + peduncolusSize;
			var l3 = new HLine(l3size, currentY, currentX);
			ndo.append(l3);

			// L4
			var l4;
			if (l2_4size < 0) {
				l4 = new VLine(-l2_4size, currentY + l2_4size, currentX);
			}
			else {
				l4 = new VLine(l2_4size, currentY, currentX);
			}
			ndo.append(l4);

			currentY = currentY + l2_4size;

			// L5
			if (peduncolusSize > 0) {
				var l5 = new HLine(peduncolusSize, currentY, currentX);
				currentX = currentX + peduncolusSize;
				ndo.append(l5);
			}
		}
		else {
			//L1
			currentX = currentX + rectFrom.width;
			var l1 = new HLine(peduncolusSize, currentY, currentX);
			ndo.append(l1);

			currentX += peduncolusSize;

			//L2
			var l2Size = ((rectTo.top + rectTo.height / 2) - (rectFrom.top + rectFrom.height / 2)) + 7;
			var l2;
			if (l2Size < 0) {
				l2 = new VLine(-l2Size, currentY + l2Size, currentX);
			}
			else {
				l2 = new VLine(l2Size, currentY, currentX);
			}
			ndo.append(l2);
			currentY = currentY + l2Size;

			//L3
			var l3 = new HLine(((rectTo.left - (rectFrom.left + rectFrom.width)) - peduncolusSize), currentY, currentX);
			ndo.append(l3);
		}

		//arrow
		var arr = $("<img src='" + Otime.baseUrl + "/assets/img/linkArrow.png'>").css({
			position: 'absolute',
			top: rectTo.top + (rectTo.height / 2) + 2,
			left: rectTo.left - 5
		});

		ndo.append(arr);

		return ndo;
	};

	self.drawLink = function (from, to, type) {
		var tlg = $(self.parentId + " #ganttC .task-list-group");
		var links = $(".ganttLinks", tlg);
		links.css({
			width: computedTableWidth
		});

		var ndo = getNodeBox(from, to);

		var rectFrom = buildRect(from);
		var rectTo = buildRect(to);

		if (!rectFrom || !rectTo) {
			return false;
		}

		switch (type) {
			case "SS":
				self.element.find(".ganttLinks").append(drawStartToStart(ndo, rectFrom, rectTo, peduncolusSize));
				break;
			case "SF":
				self.element.find(".ganttLinks").append(drawStartToFinish(ndo, rectTo, rectFrom, peduncolusSize));
				break;
			case "FF":
				self.element.find(".ganttLinks").append(drawFinishToFinish(ndo, rectFrom, rectTo, peduncolusSize));
				break;
			case "FS":
			default:
				self.element.find(".ganttLinks").append(drawFinishToStart(ndo, rectFrom, rectTo, peduncolusSize));
				break;
		}
	};

	self.refreshGantt = function (dt) {
		if (!self.zoom) {
			//guess the zoom level in base of period
			var days = (self.originalEndMillis.getTime() - self.originalStartMillis.getTime()) / (3600000 * 24);
			self.zoom = self.zoomLevels[days < 2 ? 0 : (days < 15 ? 1 : (days < 60 ? 2 : (days < 150 ? 3 : (days < 400 ? 4 : 5))))];
		}

		$("#gantt-tooltip").css({ opacity: 0, left: 0, top: 0 });
		self.init();
		Otime.Tasks.applyLinks(true);
		//		setTimeout(function(){
		self.centerOnToday(dt);
		//		}, 500);

		return self;
	};

	self.centerOnToday = function (date) {		
		date = date ? new Date(date).getTime() : new Date().getTime();
		var x = Math.round((date - self.startMillis.getTime()) * self.fx);
		$("#scr3").scrollLeft(x - $("#scr3").width() / 2);
		return self;
	};

	self.centerOnData = function () {		
		self.centerOnToday(self.minDate);
		var today = new Date();
		if (today < self.minDate) {
			return self.centerOnToday(self.minDate);
		} else if (today > self.maxDate) {
			return self.centerOnToday(self.maxDate);
		}

		return self.centerOnToday();
	};

	return self;
}
